# This is my very first attempt at using Blenders Python API and I am also not a Python dev so I apologize for the very messy code!
import bpy

bl_info = {
      "name": "Easy Clouds",
      "author": "LunarKitten",
      "version": (2, 0, 0),
      "blender": (4, 3, 2),
      "location": "View3d > Tool",
      "warning": "",
      "wiki_url": "https://codeberg.org/LunarKitten/EasyClouds",
      "category": "Add Mesh, Material, Shader Group",
}

def new_box(x, y, z):
      x = x / 2
      y = y / 2
      z = z / 2

      verts = [(-x, -y, -z), (x, -y, -z), (x, y, -z), (-x, y, -z), (-x, -y, z), (x, -y, z), (x, y, z), (-x, y, z)]
      faces = [(3, 2, 1, 0), (4, 5, 6, 7), (3, 7, 6, 2), (2, 6, 5, 1), (1, 5, 4, 0), (0, 4, 7, 3)]

      mesh = bpy.data.meshes.new("EasyClouds Cloud Box")
      object = bpy.data.objects.new("EasyClouds Cloud Box", mesh)

      bpy.context.collection.objects.link(object)
      mesh.from_pydata(verts, [], faces)
      mesh.update(calc_edges = True)

      return object

# _inputs must be an dictionary of {name, hidden (optional), value (optional), min (optional), max (optional), socket type (optional), subtype(optional)}
# _outputs must be an array of [name, socket type (optional)]
def create_group(_name, _inputs, _outputs):
      group = bpy.data.node_groups.new(_name, 'ShaderNodeTree')

      for inp in _inputs:
            st = 'NodeSocketFloat'
            if 'socket type' in inp:
                  st = inp['socket type']
            soc = group.interface.new_socket(inp['name'], in_out = 'INPUT', socket_type = st)
            if 'subtype' in inp:
                  soc.subtype = inp['subtype']
            if 'hidden' in inp:
                  soc.hide_value = inp['hidden']
            if 'value' in inp:
                  soc.default_value = inp['value']
            if 'min' in inp:
                  soc.min_value = inp['min']
            if 'max' in inp:
                  soc.max_value = inp['max']

      for out in _outputs:
            soc = group.interface.new_socket(out[0], in_out = 'OUTPUT', socket_type = 'NodeSocketFloat')

            if len(out) >= 2:
                  soc.socket_type = out[1]

      return group

def new_node(_name, _cnt, _operation = None, _group = None):
      node = _cnt.new(_name)

      if _operation != None:
            node.operation = _operation

      if _name == 'ShaderNodeGroup':
            node.node_tree = bpy.data.node_groups[_group.name]
            
      return node

# Sort nodes into an automatic table like layout and link them based on their table number 
# _nodes must be an array of [node reference, column, height (optional), width (optional)] in column order
# Create spacer nodes by using None instead of a node reference
# _links must be an array of [node index, output number, node index, input number]
# _link must be a reference to link method
# _values must be an array of [node index, input number, value]
def table_nodes(_nodes, _links = [], _link = None, _values = []):
      separation_height = 20
      separation_width = 50
      # The number of widths = the number of columns
      widths = []
      heights = [[]]
      total_height = 0
      nodes = []
      for node in _nodes:
            # Add the node reference to its own array
            nodes.append(node[0])

            column = node[1]
            height = 140
            width = 140

            # If the node has custom width or height grab it here
            if len(node) >= 3:
                  height = node[2]
                  if len(node) >= 4:
                        width = node[3]

            # Create new width entry if one does not exist for this column
            if len(widths) <= column:
                  widths.append(0)
                  # If a new width is needed a new height is too
                  heights.append([])
                  heights[column].append([0, 0]) # First element of each height row contains the number of nodes, and total height

            # Check if the nodes width is larger then the columns width and set the columns width accordingly 
            if widths[column] <= width:
                  widths[column] = width 

            # Add a new count to height, increase the total height, and add a new height
            heights[column][0][0] += 1
            heights[column].append(heights[column][0][1])
            heights[column][0][1] += height + separation_height
            # Check if a new total height is needed
            if total_height < heights[column][0][1]:
                  total_height = heights[column][0][1]
      
      # Remove extra separation height from total height
      total_height -= separation_height

      # Finlay loop over the heights array and set the corresponding nodes position
      index = 0
      for col_idx, column in enumerate(heights):
            for row_idx, node_height in enumerate(column):
                  if type(node_height) != int:
                        continue

                  # Calculate X position
                  x = 0
                  for i in range(col_idx + 1):
                        x += widths[i]
                  x += col_idx * separation_width

                  # Calculate Y position
                  y = (total_height - (column[0][1] - separation_height)) / 2
                  y += column[row_idx]

                  if nodes[index] != None:
                        # Set nodes label to its index
                        nodes[index].label = str(index)
                        # Set the nodes position
                        nodes[index].location = (x, y * -1)

                  # Iterate index
                  index += 1

      # Link nodes together based on _links array
      for link in _links:
            _link(nodes[link[0]].outputs[link[1]], nodes[link[2]].inputs[link[3]])

      # Set new node values
      for value in _values:
            if type(value[1]) == bool:
                  nodes[value[0]].use_clamp = value[1]
                  continue 
            
            nodes[value[0]].inputs[value[1]].default_value = value[2]

      return nodes

def cloud_shader():
      group = create_group('Cloud Shader', 
            [
                  {'name': 'Color', 'value': (1, 1, 1, 1), 'socket type': 'NodeSocketColor'}, 
                  {'name': 'Texture', 'value': 1, 'hidden': True}, 
                  {'name': 'Density', 'value': 200, 'min': 0}
            ], [
                  ['Volume', 'NodeSocketShader']
            ])
      cnt = group.nodes
      link = group.links.new
      
      table_nodes([
            [new_node('NodeGroupInput', cnt),               0],
            [new_node('ShaderNodeGamma', cnt),              1, 100],
            [new_node('ShaderNodeMath', cnt, 'MULTIPLY'),   1],
            [new_node('ShaderNodeLightPath', cnt),          1, 300],
            [None,                                          2], 
            [new_node('ShaderNodeMath', cnt, 'POWER'),      2],
            [new_node('ShaderNodeMath', cnt, 'ADD'),        2],
            [new_node('ShaderNodeMath', cnt, 'MULTIPLY'),   3],
            [None,                                          3], 
            [new_node('ShaderNodeMath', cnt, 'POWER'),      3],
            [new_node('ShaderNodeVolumeScatter', cnt),      4],
            [new_node('NodeGroupOutput', cnt),              5],
      ],
      [
            [0, 0, 10, 0],
            [0, 1, 1, 0],
            [0, 2, 2, 1],
            [1, 0, 2, 0],
            [2, 0, 7, 0],
            [3, 8, 5, 1],
            [3, 8, 6, 0],
            [5, 0, 7, 1],
            [6, 0, 9, 1],
            [7, 0, 10, 1],
            [9, 0, 10, 2],
            [10, 0, 11, 0],
      ], link,
      [
            [5, 0, 0.54],
            [6, 1, 1],
            [9, 0, 0.9],
      ])

      return group

def cloud_texture_combine():
      group = create_group('Cloud Combine', 
            [
                  {'name': 'Bounding Map', 'hidden': True}, 
                  {'name': 'Shape Texture', 'hidden': True}, 
                  {'name': 'Shape Size', 'value': 1}, 
                  {'name': 'Detail Texture', 'hidden': True}, 
                  {'name': 'Detail End', 'min': 0, 'value': 0.05}, 
                  {'name': 'Detail Threshold', 'value': 0.05, 'min': 0, 'max': 1, 'subtype': 'FACTOR'}, 
            ], [
                  ['Fac'],
                  ['Shape'],
                  ['Detail'],
                  ['Visualized'],
            ])
      cnt = group.nodes
      link = group.links.new

      table_nodes([
            [new_node('NodeGroupInput', cnt),                     0],
            [new_node('ShaderNodeMath', cnt, 'MULTIPLY'),         1],
            [new_node('ShaderNodeMath', cnt, 'SUBTRACT'),         1],
            [new_node('ShaderNodeMath', cnt, 'GREATER_THAN'),     2],
            [new_node('ShaderNodeMath', cnt, 'GREATER_THAN'),     2],
            [new_node('ShaderNodeMapRange', cnt),                 2, 300],
            [None,                                                3],
            [new_node('ShaderNodeMath', cnt, 'ADD'),              3],
            [new_node('ShaderNodeMath', cnt, 'LESS_THAN'),        3],
            [new_node('ShaderNodeMath', cnt, 'MULTIPLY'),         3],
            [None,                                                4, 200],
            [new_node('ShaderNodeMath', cnt, 'DIVIDE'),           4],
            [None,                                                4, 200],
            [new_node('ShaderNodeMath', cnt, 'MULTIPLY'),         4],
            [None,                                                5],
            [None,                                                5],
            [None,                                                5],
            [None,                                                5],
            [None,                                                5],
            [new_node('ShaderNodeMath', cnt, 'GREATER_THAN'),     5],
            [new_node('ShaderNodeMath', cnt, 'ADD'),              6],
            [None,                                                6],
            [None,                                                6],
            [new_node('NodeGroupOutput', cnt),                    7],
      ],
      [
            [0, 0, 1, 0],
            [0, 1, 1, 1],
            [0, 2, 2, 0],
            [0, 2, 3, 1],
            [0, 2, 5, 2],
            [0, 3, 13, 1],
            [0, 4, 2, 1],
            [0, 5, 19, 1],
            [1, 0, 3, 0],
            [1, 0, 4, 0],
            [1, 0, 5, 0],
            [2, 0, 4, 1],
            [2, 0, 5, 1],
            [3, 0, 20, 0],
            [3, 0, 7, 0],
            [3, 0, 23, 1],
            [4, 0, 7, 1],
            [5, 0, 8, 0],
            [5, 0, 9, 1],
            [7, 0, 11, 0],
            [8, 0, 9, 0],
            [9, 0, 13, 0],
            [11, 0, 23, 3],
            [13, 0, 19, 0],
            [19, 0, 20, 1],
            [19, 0, 23, 2],
            [20, 0, 23, 0],
      ], link,
      [
            [8, True],
            [8, 1, 1],
            [9, True],
            [11, True],
            [11, 1, 2],
            [19, True],
      ])

      return group

def cloud_bound():
      group = create_group('Cloud Bound', 
            [
                  {'name': 'Vector', 'hidden': True, 'socket type': 'NodeSocketVector'}, 
                  {'name': 'Z Mid',       'value': 0.2, 'min': 0, 'max': 1, 'subtype': 'FACTOR'}, 
                  {'name': 'Z Power',     'value': 1, 'min': 0}, 
                  {'name': 'XY Power',    'value': 1, 'min': 0}, 
            ], [
                  ['Fac'],
            ])
      cnt = group.nodes
      link = group.links.new

      nodes = table_nodes([
            [new_node('NodeGroupInput', cnt),                     0],
            [new_node('ShaderNodeVectorMath', cnt),               1, 200],
            [new_node('ShaderNodeSeparateXYZ', cnt),              1],
            [new_node('ShaderNodeVectorMath', cnt, 'MULTIPLY'),   2, 200],
            [new_node('ShaderNodeMapRange', cnt),                 2, 240],
            [new_node('ShaderNodeMapRange', cnt),                 2, 240],
            [new_node('ShaderNodeTexGradient', cnt),              3, 400],
            [new_node('ShaderNodeMath', cnt, 'MULTIPLY'),         3],
            [new_node('ShaderNodeMath', cnt, 'POWER'),            4, 400],
            [new_node('ShaderNodeMath', cnt, 'POWER'),            4],
            [new_node('ShaderNodeMath', cnt, 'MULTIPLY'),         5],
            [new_node('NodeGroupOutput', cnt),                    6],
      ],
      [
            [0, 0, 1, 0],
            [0, 0, 2, 0],
            [0, 1, 4, 2],
            [0, 1, 5, 2],
            [0, 2, 9, 1],
            [0, 3, 8, 1],
            [1, 0, 3, 0],
            [2, 2, 4, 0],
            [2, 2, 5, 0],
            [3, 0, 6, 0],
            [4, 0, 7, 0],
            [5, 0, 7, 1],
            [6, 0, 8, 0],
            [7, 0, 9, 0],
            [8, 0, 10, 0],
            [9, 0, 10, 1],
            [10, 0, 11, 0],
      ], link,
      [
            [1, 1, (-0.5, -0.5, 0)],
            [3, 1, (2, 2, 0)],
            [5, 1, 1],
            [7, True],
            [8, True],
            [9, True],
            [10, True],
      ])

      nodes[6].gradient_type = 'SPHERICAL'

      return group

def layer_bound():
      group = create_group('Layer Bound', 
            [
                  {'name': 'Vector', 'hidden': True, 'socket type': 'NodeSocketVector'}, 
                  {'name': 'Z Mid',       'value': 0.2, 'min': 0, 'max': 1, 'subtype': 'FACTOR'}, 
                  {'name': 'Z Power',     'value': 1, 'min': 0}, 
                  {'name': 'XY Power',    'value': 1, 'min': 0}, 
            ], [
                  ['Fac'],
            ])
      cnt = group.nodes
      link = group.links.new

      table_nodes([
            [new_node('NodeGroupInput', cnt),                     0],
            [new_node('ShaderNodeSeparateXYZ', cnt),              1],
            [new_node('ShaderNodeMath', cnt, 'PINGPONG'),         2],
            [new_node('ShaderNodeMath', cnt, 'PINGPONG'),         2],
            [new_node('ShaderNodeMapRange', cnt),                 2, 240],
            [new_node('ShaderNodeMapRange', cnt),                 2, 240],
            [new_node('ShaderNodeMath', cnt, 'MULTIPLY'),         3, 400],
            [new_node('ShaderNodeMath', cnt, 'MULTIPLY'),         3],
            [new_node('ShaderNodeMapRange', cnt),                 4, 600],
            [new_node('ShaderNodeMath', cnt, 'POWER'),            4],
            [new_node('ShaderNodeMath', cnt, 'POWER'),            5, 400],
            [new_node('ShaderNodeMath', cnt, 'MULTIPLY'),         6],
            [new_node('NodeGroupOutput', cnt),                    7],
      ],
      [
            [0, 0, 1, 0],
            [0, 1, 4, 2],
            [0, 1, 5, 2],
            [0, 2, 9, 1],
            [0, 3, 10, 1],
            [1, 0, 2, 0],
            [1, 1, 3, 0],
            [1, 2, 4, 0],
            [1, 2, 5, 0],
            [2, 0, 6, 0],
            [3, 0, 6, 1],
            [4, 0, 7, 0],
            [5, 0, 7, 1],
            [6, 0, 8, 0],
            [7, 0, 9, 0],
            [8, 0, 10, 0],
            [9, 0, 11, 1],
            [10, 0, 11, 0],
            [11, 0, 12, 0],
      ], link,
      [
            [2, True],
            [3, True],
            [5, 1, 1],
            [6, True],
            [7, True],
            [8, 2, 0.05],
            [9, True],
            [10, True],
            [11, True],
      ])

      return group

def cloud_shape():
      group = create_group('Cloud Shape', 
            [
                  {'name': 'Vector', 'hidden': True, 'socket type': 'NodeSocketVector'}, 
                  {'name': 'Scale',       'value': 5        }, 
                  {'name': 'Detail',      'value': 0,       'min': 0, 'max': 15}, 
                  {'name': 'Roughness',   'value': 0.5,     'min': 0, 'max': 1, 'subtype': 'FACTOR'}, 
                  {'name': 'Lacunarity',  'value': 2,       'min': 0}, 
                  {'name': 'Random',      'value': 1,       'min': 0, 'max': 1, 'subtype': 'FACTOR'}, 
                  {'name': 'Brightness',  'value': 0,       'min': -100, 'max': 100}, 
                  {'name': 'Contrast',    'value': 0,       'min': -100, 'max': 100}, 
            ], [
                  ['Fac'],
            ])
      cnt = group.nodes
      link = group.links.new

      nodes = table_nodes([
            [new_node('NodeGroupInput', cnt),                     0],
            [new_node('ShaderNodeTexVoronoi', cnt),               1, 200],
            [new_node('ShaderNodeMath', cnt, 'MULTIPLY'),         2],
            [new_node('ShaderNodeMath', cnt, 'ADD'),              2],
            [new_node('ShaderNodeBrightContrast', cnt),           3],
            [new_node('NodeGroupOutput', cnt),                    4],
      ],
      [
            [0, 0, 1, 0],
            [0, 1, 1, 2],
            [0, 2, 1, 3],
            [0, 3, 1, 4],
            [0, 4, 1, 5],
            [0, 5, 1, 8],
            [0, 6, 4, 1],
            [0, 7, 4, 2],
            [1, 0, 2, 0],
            [2, 0, 3, 0],
            [3, 0, 4, 0],
            [4, 0, 5, 0],
      ], link,
      [
            [2, 1, -1],
            [3, 1, 1],
      ])

      nodes[1].normalize = True

      return group

def cloud_detail():
      group = create_group('Cloud Detail', 
            [
                  {'name': 'Detail Vector', 'hidden': True, 'socket type': 'NodeSocketVector'}, 
                  {'name': 'Scale',       'value': 5        }, 
                  {'name': 'Detail',      'value': 2,       'min': 0, 'max': 15}, 
                  {'name': 'Roughness',   'value': 0.5,     'min': 0, 'max': 1, 'subtype': 'FACTOR'}, 
                  {'name': 'Lacunarity',  'value': 2,       'min': 0}, 
                  {'name': 'Distortion',  'value': 0        }, 
                  {'name': 'Brightness',  'value': 0,       'min': -100, 'max': 100}, 
                  {'name': 'Contrast',    'value': 0,       'min': -100, 'max': 100}, 
                  {'name': 'Density Vector', 'hidden': True, 'socket type': 'NodeSocketVector'}, 
                  {'name': 'Scale',       'value': 5        }, 
                  {'name': 'Detail',      'value': 2,       'min': 0, 'max': 15}, 
                  {'name': 'Roughness',   'value': 0.5,     'min': 0, 'max': 1, 'subtype': 'FACTOR'}, 
                  {'name': 'Lacunarity',  'value': 2,       'min': 0}, 
                  {'name': 'Distortion',  'value': 0        }, 
                  {'name': 'Brightness',  'value': 0,       'min': -100, 'max': 100}, 
                  {'name': 'Contrast',    'value': 0,       'min': -100, 'max': 100}, 
            ], [
                  ['Fac'],
            ])
      cnt = group.nodes
      link = group.links.new

      table_nodes([
            [new_node('NodeGroupInput', cnt),                     0, 400],
            [new_node('ShaderNodeTexNoise', cnt),                 1, 280],
            [new_node('ShaderNodeTexNoise', cnt),                 1, 280],
            [new_node('ShaderNodeBrightContrast', cnt),           2],
            [new_node('ShaderNodeBrightContrast', cnt),           2],
            [new_node('ShaderNodeMath', cnt, 'SUBTRACT'),         3],
            [new_node('NodeGroupOutput', cnt),                    4],
      ],
      [
            [0, 0, 1, 0],
            [0, 1, 1, 2],
            [0, 2, 1, 3],
            [0, 3, 1, 4],
            [0, 4, 1, 5],
            [0, 5, 1, 8],
            [0, 6, 3, 1],
            [0, 7, 3, 2],
            [0, 8, 2, 0],
            [0, 9, 2, 2],
            [0, 10, 2, 3],
            [0, 11, 2, 4],
            [0, 12, 2, 5],
            [0, 13, 2, 8],
            [0, 14, 4, 1],
            [0, 15, 4, 2],
            [1, 0, 3, 0],
            [2, 0, 4, 0],
            [3, 0, 5, 0],
            [4, 0, 5, 1],
            [5, 0, 6, 0],
      ], link,
      [
            [5, True],
      ])

      return group

def cloud_material():
      material = bpy.data.materials.new(name = "Cloud")
      material.use_nodes = True
      
      cnt = material.node_tree.nodes
      link = material.node_tree.links.new

      # Remove default nodes
      cnt.remove(cnt.get('Principled BSDF'))
      cnt.remove(cnt.get('Material Output'))

      detail = cloud_detail()
      shape = cloud_shape()
      bound = cloud_bound()
      combine = cloud_texture_combine()
      shader = cloud_shader()

      table_nodes([
            [new_node('ShaderNodeTexCoord', cnt),                             0, 700],
            [None,                                                            1],
            [new_node('ShaderNodeVectorMath', cnt),                           1, 200],
            [new_node('ShaderNodeVectorMath', cnt),                           1, 200],
            [new_node('ShaderNodeVectorMath', cnt),                           1, 200],
            [new_node('ShaderNodeGroup', cnt, _group = bound),                2, 160],
            [new_node('ShaderNodeGroup', cnt, _group = shape),                2, 260],
            [new_node('ShaderNodeGroup', cnt, _group = detail),               2, 500],
            [new_node('ShaderNodeGroup', cnt, _group = combine),              3, 260],
            [new_node('ShaderNodeGroup', cnt, _group = shader),               4],
            [new_node('ShaderNodeOutputMaterial', cnt),                       5],
      ], [
            [0, 3, 2, 0],
            [0, 3, 3, 0],
            [0, 3, 4, 0],
            [0, 3, 5, 0],
            [2, 0, 6, 0],
            [3, 0, 7, 0],
            [4, 0, 7, 8],
            [0, 0, 5, 0],
            [5, 0, 8, 0],
            [6, 0, 8, 1],
            [7, 0, 8, 3],
            [8, 0, 9, 1],
            [9, 0, 10, 1],
      ], link,
      [
            [2, 1, (1.2, -0.38, 0)],
            [5, 1, 0.15],
            [5, 2, 1.11],
            [5, 3, 1.24],
            [6, 1, 0.962],
            [6, 2, 4],
            [6, 3, 0.381],
            [6, 4, 3.21],
            [6, 6, -1.86],
            [6, 7, 10.85],
            [7, 1, 25],
            [7, 2, 3.19],
            [7, 3, 0.8],
            [7, 5, 1.9],
            [7, 6, 0.07],
            [7, 7, 15.27],
            [7, 9, 2.5],
            [7, 10, 11.83],
            [7, 11, 0.433],
            [7, 14, 5.33],
            [7, 15, -43.45],
            [8, 2, 0.29],
            [8, 4, 0.05],
            [8, 5, 0.066],
      ])


      return material 

def cloud_layer_material():
      material = bpy.data.materials.new(name = "Cloud Layer")
      material.use_nodes = True
      
      cnt = material.node_tree.nodes
      link = material.node_tree.links.new

      # Remove default nodes
      cnt.remove(cnt.get('Principled BSDF'))
      cnt.remove(cnt.get('Material Output'))

      detail = cloud_detail()
      shape = cloud_shape()
      layer = layer_bound()
      combine = cloud_texture_combine()
      shader = cloud_shader()

      table_nodes([
            [new_node('ShaderNodeTexCoord', cnt),                             0, 700],
            [None,                                                            1],
            [new_node('ShaderNodeVectorMath', cnt),                           1, 200],
            [new_node('ShaderNodeVectorMath', cnt),                           1, 200],
            [new_node('ShaderNodeVectorMath', cnt),                           1, 200],
            [new_node('ShaderNodeGroup', cnt, _group = layer),                2, 160],
            [new_node('ShaderNodeGroup', cnt, _group = shape),                2, 260],
            [new_node('ShaderNodeGroup', cnt, _group = detail),               2, 500],
            [new_node('ShaderNodeGroup', cnt, _group = combine),              3, 260],
            [new_node('ShaderNodeGroup', cnt, _group = shader),               4],
            [new_node('ShaderNodeOutputMaterial', cnt),                       5],
      ], [
            [0, 3, 2, 0],
            [0, 3, 3, 0],
            [0, 3, 4, 0],
            [0, 3, 5, 0],
            [2, 0, 6, 0],
            [3, 0, 7, 0],
            [4, 0, 7, 8],
            [0, 0, 5, 0],
            [5, 0, 8, 0],
            [6, 0, 8, 1],
            [7, 0, 8, 3],
            [8, 0, 9, 1],
            [9, 0, 10, 1],
      ], link,
      [
            [5, 2, 0.99],
            [5, 3, 0.25],
            [6, 1, 0.577],
            [6, 2, 4],
            [6, 3, 0.483],
            [6, 4, 3.57],
            [6, 6, -0.26],
            [6, 7, 4.11],
            [7, 1, 12.75],
            [7, 2, 3.19],
            [7, 3, 0.8],
            [7, 5, 1.9],
            [7, 6, 0.07],
            [7, 7, 15.27],
            [7, 9, 1.19], 
            [7, 10, 11.83], 
            [7, 11, 0.433], 
            [7, 14, 1.71], 
            [7, 15, 1.02], 
            [8, 2, 1.44], 
            [8, 4, 0.03], 
            [8, 5, 0.032], 
      ])

      return material 
      
# Side Panel Buttons
class easyclouds_panel(bpy.types.Panel):
      bl_label = "Easy Clouds"
      bl_idname = "EASYCLOUDS_PT_panel"
      bl_space_type = 'VIEW_3D'
      bl_region_type = 'UI'
      bl_category = 'Tool'

      def draw(self, context):
            layout = self.layout

            row = layout.row()
            row.label(text = 'Use edit mode to show more/less of the clouds', icon = 'INFO')
            
            row = layout.row()
            row.operator("easyclouds.cloud")

            row = layout.row()
            row.operator("easyclouds.layer")

class create_cloud(bpy.types.Operator):
      bl_label = "Create Cloud"
      bl_idname = "easyclouds.cloud"

      def execute(self, context):
            object = new_box(1, 1.5, 0.8)
            object.active_material = cloud_material()

            return {'FINISHED'}

class create_cloud_layer(bpy.types.Operator):
      bl_label = "Create Layer"
      bl_idname = "easyclouds.layer"

      def execute(self, context):
            object = new_box(10, 10, 1)
            object.active_material = cloud_layer_material()

            return {'FINISHED'}









# -- Older code left for reference -- #

# group = bpy.data.node_groups.new('EasyClouds Invert Range Clamped', 'ShaderNodeTree')
# cnt = group.nodes
# link = group.links.new
# # Final Material
# cnt = material.node_tree.nodes
# link = material.node_tree.links.new
# link(n_tc.outputs[3], n_vm.inputs[0])
# n_vm.inputs[1].default_value = (1, 1, 1)








# Register Unregister
classes = [easyclouds_panel, create_cloud, create_cloud_layer]

def register():
      for cls in classes:
            bpy.utils.register_class(cls)

def unregister():
      for cls in classes:
            bpy.utils.unregister_class(cls)

if __name__ == "__main__":
    register()